#####################################################################################
#
#		:::[  T h e   R u s s i a n   E g g d r o p  R e s o u r c e  ]::: 
#      ____                __                                                      
#     / __/___ _ ___ _ ___/ /____ ___   ___      ___   ____ ___ _     ____ __ __   
#    / _/ / _ `// _ `// _  // __// _ \ / _ \    / _ \ / __// _ `/    / __// // /   
#   /___/ \_, / \_, / \_,_//_/   \___// .__/ __ \___//_/   \_, / __ /_/   \___/    
#        /___/ /___/                 /_/    /_/           /___/ /_/                
#
#
#####################################################################################
#
# gismeteo.tcl 1.6
#
# Description:
#	Displays weather forecast from gismeteo.ru
#
# Authors: Shrike <shrike@eggdrop.org.ru>
#
# Official support: irc.eggdrop.org.ru @ #eggdrop
# 
#####################################################################################
#
# Version History:
#	
#  		SHR - Shrike
#	
#		Who		DATE	Changes
#   	---     ------  -------------------------------------------------------------
#
# v1.6.1
#		SHR		040927	More accurate fix for temperature signs.
# v1.6	
#		SHR		040927	Fixed minor bug when sign was "" ... and there is no weather
#						displayed.
#
#						Possible added new feature "weather for next day", in 
#						gismeteo flash engine it's built in url scheme... but currently
#						not realized. maybe cause they not rewrited their flash banners.
#						so... i added feature to display weather for tomorrow, but not
#						changed help. !weather  city will try to use second
#						weather parameters from gismeteo.
#
# v1.5
#		SHR		040701	Added missed sky types.
#						Modified weather(agent). Now we introduce yourself as GismeteoBar
#						Added random numbers to request... flash version does it. I don't
#						know why, but think it would be better...
#						Fixed bug with "wrong # args: should be "::weather::alias_on_error"
#						(missed argument) ;)
#						Added checking if script already in update-loop...
#
#		SHR		040623	Fixed bug with koi8-r. City displayed in wrong encoding.
#						(thanx to Zl0y for bug report)
# v1.4			
#		SHR		040618	Added additional bind "!"
#						Due a testing found a bug in weatherupdate, was incorrect of
#						city. fixed.
#						Added shortcuts
#
#   	SHR		040617	Added this header ;)
#						Refacored for namespace usage
#						Removed old socket functions
#						Rewrited to use socket functions in egglib_pub
#						Added proxy support
#						Added support for koi8-r bots
#						Added additional bind "!w"
#					
#####################################################################################
# Last version of egglib_pub can be grabbed here:
# http://eggdrop.org.ru/scripts/egglib.zip

if { ![info exists egglib(ver)] } {
	putlog "***********************************************"
	putlog "             egglib_pub NOT FOUND !"
	putlog "   Download last version of egglib_pub here:"
	putlog "  http://eggdrop.org.ru/scripts/egglib_pub.zip"
	putlog "***********************************************"	
	die
}

if { [expr {$egglib(ver) < 1.4}] } {
	putlog "***********************************************"
	putlog " YOUR VERSION OF egglib_pub IS TOO OLD !"
	putlog "   Download last version of egglib_pub here:"
	putlog "  http://eggdrop.org.ru/scripts/egglib_pub.zip"
	putlog "***********************************************"
	putlog " version installed : $egglib(ver)"
	putlog " version required: 1.4"
	die
}

namespace eval weather {}

setudef flag nopubweather

bind pub - !		 		::weather::pub_weather
bind msg - !		 		::weather::msg_weather
bind pub - !w		 		::weather::pub_weather
bind msg - !w		 		::weather::msg_weather
bind pub - !weather 		::weather::pub_weather
bind msg - !weather 		::weather::msg_weather
bind pub - ! 			::weather::pub_weather
bind msg - ! 			::weather::msg_weather
bind pub - !pogoda 			::weather::pub_weather
bind msg - !pogoda 			::weather::msg_weather
bind pub - !temp 			::weather::pub_weather
bind msg - !temp 			::weather::msg_weather

bind dcc n weatherupdate   	::weather::dcc_weatherupdate
bind dcc n weatherupd   	::weather::dcc_weatherupdate
bind dcc n wupd   			::weather::dcc_weatherupdate

foreach p [array names weather *] { catch {unset weather($p) } }

set weather(timeout) 	20
set weather(debug) 		0

set weather(use_proxy) 	0
set weather(proxy_host) "proxy1.bezeqint.net"
set weather(proxy_port) 3128

set weather(file) 		"weather.dat"

set weather(cp1251) 	1

set weather(ver) 		"1.6.1"
set weather(authors) 	"Shrike <shrike@eggdrop.org.ru>"
set weather(agent)		"Internet MDB Client"

set weather(shortcuts_in) {
  "" "" "" 
  "" "" "" 
  "" "kzn" "spb"
  "msk" "nsk" ""
  
}

set weather(shortcuts_out) {
  "-" "-" "" 
  "" "" "" 
  "" "" "-"
  "" "" ""
}

proc ::weather::msg_weather { nick uhost hand args } {
	if { [llength [split [lindex $args 0] ] ] > 1 || [llength [split [lindex $args 0] ] ] == 0 } { set args [split [lindex $args 0]] }
	::weather::weather $nick $uhost $nick $nick $args
}

proc ::weather::pub_weather { nick uhost hand chan args } {
	if {[channel get $chan nopubweather]} return
	if { [llength [split [lindex $args 0] ] ] > 1 || [llength [split [lindex $args 0] ] ] == 0 } { set args [split [lindex $args 0]] }
	::weather::weather $nick $uhost $hand $chan $args
}

########################################################
proc ::weather::weather { nick uhost hand chan args } {
 	global weather
 	set weather(cities) [::egglib::readdata $weather(file)]
 	if { [llength [split [lindex $args 0] ] ] > 1 || [llength [split [lindex $args 0] ] ] == 0 } { set args [split [lindex $args 0]] }
	if {[llength $args]==0} { ::egglib::outh $nick $chan "!" "<>"; return }
	if { [regexp -nocase -- {^$} [::egglib::tolower [lindex $args 0] ] ]} { set s 1; set nd 1 } else { set s 0; set nd 0 }
	set query [::egglib::backslash [::egglib::tolower [::egglib::join_args $args $s end]]]
	::egglib::log $chan $nick "weather" "$query"
	set num [lsearch -regexp $weather(shortcuts_in) "^($query)$" ]
	if {[expr {$num > -1}]} { set query [lindex $weather(shortcuts_out) $num] }
	if {[info exists weather(host,$uhost)]} { ::egglib::outn $nick "    ..."; return }
	set num [lsearch -regexp $weather(cities) "^($query)\ \#(.*?)$"]
	if {[expr $num == -1] || ![regexp -- {^.*?\ \#(\d+)$} [lindex $weather(cities) $num] garb wid] } { ::egglib::out $nick $chan "        ..."; return }
	set weather(host,$uhost) 1
	set url "http://img.gismeteo.ru/flashinf/FLA$wid.TXT?[rand 9999]"
	set id [::egglib::http_init "::weather::"]
	if { $weather(use_proxy) } { ::egglib::http_set_proxy $id $weather(proxy_host) $weather(proxy_port) }
	::egglib::http_set_timeout $id $weather(timeout)
  	::egglib::http_get $id $url [list $nick $chan $uhost $nd]
}

proc ::weather::on_data {id html nick chan uhost nd} {
	global weather
	catch {unset weather(host,$uhost)}
  	if { $weather(debug) } { putlog "\[weather\] callback executed... analyzing data..." }
  	set wp1 ""; set wp2 ""; set s1 ""; set s2 ""; set wt ""	
  	foreach line [split $html \n] { 
  		if { $nd } {
  			if { [regexp -nocase -- {\&town\=(.*?)\&townen\=.*?\&daten\=(\d+)\&monthn\=(\d+)\&dayn\=(\d+)\&wparam1n\=(.*?)\&wparam2n\=(.*?)\&s1n\=(.*?)\&s2n\=(.*?)\&wtn\=(.*?)\&} $line gr city md mm wd wp1 wp2 s1 s2 wt] } { break }
  		} else {
  			if { [regexp -nocase -- {\&town\=(.*?)\&townen\=.*?\&date\=(\d+)\&month\=(\d+)\&day\=(\d+)\&wparam1\=(.*?)\&wparam2\=(.*?)\&s1\=(.*?)\&s2\=(.*?)\&wt\=(.*?)\&} $line gr city md mm wd wp1 wp2 s1 s2 wt] } { break }
  		}
	}
	if { $wp1 == "" || $wp2 == "" || $city == "" || $wt == "" } {	::egglib::out $nick $chan "        ..."; return }
	regsub -all -- \, $wp2 {.} wp2
	
	set city1 [encoding convertfrom cp1251 $city] 
	if { $weather(cp1251) } {
		set city [encoding convertto cp1251 $city1] 
	} else {
		set city [encoding convertto koi8-r $city1] 
	}
	if { $s1 == "" } { set s1 "32" }
	if { $s2 == "" } { set s2 "32" }
  	if { $weather(debug) } { putlog "\[weather\] wp1=$wp1 wp2=$wp2 s1=$s1 s2=$s2 wt=$wt city=$city" }
	
	set s1 [string trim [binary format "c" $s1]]
	set s2 [string trim [binary format "c" $s2]]
	set t2 [expr {($wp1 - (int($wp1/17000)*17000)) / 9}] 
	set t1 [expr {int($wp1/17000)}]
	set temp "${s1}${t1}..${s2}${t2}C"
	set hum [expr {int(($wp2 - int($wp2))*100 + 1.8)}]  
	set wp2 [expr {int($wp2)}]
	set pres [expr {int($wp2 / 10000) + 550}]
	set wp2 [expr {$wp2 - (int($wp2 / 10000) * 10000)}]  
	set ws [expr {int($wp2 / 100) - 5}]  
	set wp2 [expr {$wp2 - int($wp2 / 100) * 100}]  
	set wdind [expr {($wp2 / 7) - 3}]  
	set pres "${pres} /"
	set winds " -  -  -  -"
	if {$ws == 0} {	set wind "" } else { set wind "[lindex $winds $wdind] $ws /c" }
		if { $wt > 9 } { set wt [expr {$wt - 10}] }
		
	set skyes {
		"" 
		"" 
		"" 
		"" 
		"   " 
		"  " 
		"   " 
		"  " 
		""
	}
	set wdays {" " "" "" "" "" "" "" "C" }
	set months {" " "" "" "" "" "" "" "" "" "" "" "" "" }
		
	set sky "[lindex $skyes [expr {$wt-1}]]"
	
	::egglib::out $nick $chan "\0032   \0035$city\0032  \0036\002\002[lindex $wdays $wd]\0032,\0036 $md [lindex $months $mm]\0032 -  \00312\002\002$temp\0032  \00312\002\002$pres\0032  \00312\002\002$wind\0032  \00312\002\002$hum%\0032 $sky\003"
}

proc ::weather::on_error { id nick chan uhost nd} {
	global weather
	catch {unset weather(host,$uhost)}
	if { $weather(debug) } { putlog "\[weather\] connection timed out" }
	if { $nick == " " } { return }
	::egglib::out $nick $chan "    gismeteo.ru... -.-"
}

proc ::weather::dcc_weatherupdate {hand idx args} {
	global weather
	if { [info exists weather(updating)] && $weather(updating) } {
		putlog "Weather already updating..."
		return
	} else {
		putlog "Updating cities database..."
	}
	set weather(updating) 1
	set url "http://gen.gismeteo.ru/frcdb/cityinfr.txt"
	set id [::egglib::http_init "::weather::cityfile_"]
	if { $weather(use_proxy) } { ::egglib::http_set_proxy $id $weather(proxy_host) $weather(proxy_port) }
	::egglib::http_set_timeout $id $weather(timeout)
  	::egglib::http_get $id $url ""
}

proc ::weather::cityfile_on_data {id text} {
	global weather
  	set weather(cities) {}; set weather(update_list) {}
  	foreach line [split $text \n] {
  		if { $weather(cp1251) } { 
  			set line [encoding convertto cp1251 [encoding convertfrom cp866 $line]] 
  		} else {  
  			set line [encoding convertto koi8-r [encoding convertfrom cp866 $line]] 
  		}
  		if { [regexp {^.\d+\ +(\d+)\ .\d+\ +\d+\ +(.*)$} $line garb id city] } {
  			set weather(cities) [linsert $weather(cities) end "[::egglib::tolower $city] #$id"]
  			set weather(update_list) [linsert $weather(update_list) end "$id"]
  		}
  	}
  	set weather(update_total) [llength $weather(update_list)]
  	::egglib::writedata "$weather(file)" $weather(cities)	
	utimer 0 ::weather::alias_getnext
  	putlog "\[weather\] [llength $weather(cities)] Cities Downloaded... Getting Aliases..."
}

proc ::weather::cityfile_on_error {id} { putlog "\[weather\] Failed to get cities database... ;/" }

proc ::weather::alias_getnext { } {
	global weather
	if { [llength $weather(update_list)] == 0 } { 
		putlog "\[weather\] Aliases downloaded. Total citynames in database: [llength $weather(cities)]" 
		catch { unset weather(updating) }
		return 
	}
	set wid [lindex $weather(update_list) 0]
	set weather(update_list) [lreplace $weather(update_list) 0 0]
	putlog "\[weather\] updating alias \[[llength $weather(update_list)]:$weather(update_total)\] for city with id=$wid" 
	set url "http://img.gismeteo.ru/flashinf/FLA$wid.TXT?[rand 9999]"
	set id [::egglib::http_init "::weather::alias_"]
	if { $weather(use_proxy) } { ::egglib::http_set_proxy $id $weather(proxy_host) $weather(proxy_port) }
	::egglib::http_set_timeout $id $weather(timeout)
  	::egglib::http_get $id $url $wid
}

proc ::weather::alias_on_data {id text wid} {
	global weather
  	foreach line [split $text \n] {
  		if { $weather(cp1251) } {
  			set line [encoding convertto cp1251 [encoding convertfrom cp1251 $line]]
  		} else {
  			set line [encoding convertto koi8-r [encoding convertfrom cp1251 $line]]	
  		}
  		if { [regexp {town\=(.*?)&townen\=(.*?)&date} $line garb city0 city1] } {
  			 ::weather::alias_add $wid [::egglib::tolower $city0]
  			 ::weather::alias_add $wid [::egglib::tolower $city1]
  		}
  	}
  	utimer 0 ::weather::alias_getnext
}

proc ::weather::alias_on_error {id wid} {
	putlog "\[weather\] Failed to get alias... ;/"
	utimer 0 ::weather::alias_getnext
}

proc ::weather::alias_add {id city} {
	global weather
	set weather(cities) [::egglib::readdata "$weather(file)"]
	set num [lsearch -regexp $weather(cities) "^($city) #(.*?)$"]
    if {[expr $num > -1]} { return }
    set weather(cities) [linsert $weather(cities) end "$city #$id"]
	::egglib::writedata "$weather(file)" $weather(cities)	
}

putlog "weather.tcl v$weather(ver) by $weather(authors) loaded"
